@@ -144,7 +144,7 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 	
 	item = nautilus_menu_item_new ("NautilusBurn::write_iso",
 				       _("_Write to Disc..."),
-				       _("Write ISO image to a CD or DVD disc"),
+				       _("Write disc image to a CD or DVD disc"),
 				       "gnome-dev-cdrom");
 	g_object_set_data (G_OBJECT (item), "file", file);
 	g_object_set_data (G_OBJECT (item), "window", window);
@@ -277,6 +277,8 @@ void
 nautilus_module_initialize (GTypeModule *module)
 {
 	nautilus_burn_register_type (module);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 }
 
 void
