@@ -149,6 +149,25 @@ cd_recorder_cancel (CDRecorder *cdrecorder, gboolean skip_if_dangerous)
 	return cdrecorder->priv->dangerous;
 }
 
+static void
+insert_cd_retry (CDRecorder *cdrecorder, gboolean cancel,
+		 gboolean is_reload, gboolean send_return)
+{
+	if (cancel) {
+		/* Shouldn't be dangerous on reload */
+		cd_recorder_cancel (cdrecorder, FALSE);
+	} else if (is_reload) {
+		if (send_return) {
+			write (cdrecorder->priv->cdr_stdin, "\n", 1);
+		} else {
+			kill (cdrecorder->priv->pid, SIGUSR1);
+		}
+	} else {
+		cdrecorder->priv->result = RESULT_RETRY;
+		g_main_loop_quit (cdrecorder->priv->loop);
+	}
+}
+
 static gboolean
 growisofs_stdout_read (GIOChannel   *source,
 		      GIOCondition  condition,
@@ -221,6 +240,7 @@ cdrecord_stdout_read (GIOChannel   *source,
 	char buf[1];
 	unsigned int track, mb_written, mb_total;
 	GIOStatus status;
+	gboolean res;
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
@@ -252,7 +272,15 @@ cdrecord_stdout_read (GIOChannel   *source,
 				       percent);
 		} else if (g_str_has_prefix (line, "Re-load disk and hit <CR>") ||
 			   g_str_has_prefix (line, "send SIGUSR1 to continue")) {
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   before starting, but we try to handle it anyway, since mmc
+			   profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, TRUE, (*line == 'R'));
 		} else if (g_str_has_prefix (line, "Fixating...")) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[ACTION_CHANGED], 0,
@@ -296,6 +324,7 @@ growisofs_stderr_read (GIOChannel   *source,
 	CDRecorder *cdrecorder = (CDRecorder *) data;
 	char *line;
 	GIOStatus status;
+	gboolean res;
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
@@ -307,7 +336,15 @@ growisofs_stderr_read (GIOChannel   *source,
 	if (status == G_IO_STATUS_NORMAL && !cdrecorder->priv->expect_cdrecord_to_die) {
 		g_string_prepend (cdrecorder->priv->cdr_stderr, line);
 		if (strstr (line, "unsupported MMC profile") != NULL || (strstr (line, "already carries isofs") != NULL && strstr (line, "FATAL:") != NULL)) {
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   type before starting, but we try to handle it anyway, since mmc
+			   profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "unable to open") != NULL || strstr (line, "unable to stat") != NULL) {
 			/* This fits the "open64" and "open"-like messages */
 			cdrecorder->priv->last_error = g_strdup (_("The recorder could not be accessed"));
@@ -350,6 +387,7 @@ cdrecord_stderr_read (GIOChannel   *source,
 	CDRecorder *cdrecorder = (CDRecorder *) data;
 	char *line;
 	GIOStatus status;
+	gboolean res;
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
@@ -361,14 +399,28 @@ cdrecord_stderr_read (GIOChannel   *source,
 	if (status == G_IO_STATUS_NORMAL && !cdrecorder->priv->expect_cdrecord_to_die) {
 		g_string_prepend (cdrecorder->priv->cdr_stderr, line);
 		if (strstr (line, "No disk / Wrong disk!") != NULL) {
-			g_warning ("No disk in drive, shouldn't happen");
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   before starting, but we try to handle it anyway, since mmc
+			   profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "This means that we are checking recorded media.") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The CD has already been recorded"));
 			cdrecorder->priv->result = RESULT_ERROR;
 		} else if (strstr (line, "Cannot blank disk, aborting.") != NULL) {
-			g_warning ("Right CD type wasn't loaded, shouldn't happen");
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   type before starting, but we try to handle it anyway, since
+			   mmc profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, TRUE,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "Data may not fit on current disk") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The files selected did not fit on the CD"));
 			/* FIXME should we error out in that case?
@@ -377,9 +429,15 @@ cdrecord_stderr_read (GIOChannel   *source,
 			cdrecorder->priv->last_error = g_strdup (_("All audio files must be stereo, 16-bit digital audio with 44100Hz samples"));
 			cdrecorder->priv->result = RESULT_ERROR;
 		} else if (strstr (line, "cannot write medium - incompatible format") != NULL) {
-			g_warning ("Right CD type wasn't loaded, shouldn't happen");
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   type before starting, but we try to handle it anyway, since
+			   mmc profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
 			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "DMA speed too slow") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The system is too slow to write the CD at this speed. Try a lower speed."));
 		}
@@ -398,10 +456,8 @@ cdrecord_stderr_read (GIOChannel   *source,
 }
 
 static gboolean
-media_type_matches (CDMediaType type, gboolean *needs_blank)
+media_type_matches (CDMediaType type)
 {
-	*needs_blank = FALSE;
-
 	switch (type)
 	{
 	case CD_MEDIA_TYPE_ERROR:
@@ -416,17 +472,14 @@ media_type_matches (CDMediaType type, gboolean *needs_blank)
 	case CD_MEDIA_TYPE_CDR:
 		return TRUE;
 	case CD_MEDIA_TYPE_CDRW:
-		*needs_blank = TRUE;
 		return TRUE;
 	case CD_MEDIA_TYPE_DVDR:
 		return TRUE;
 	case CD_MEDIA_TYPE_DVDRW:
-		*needs_blank = TRUE;
 		return TRUE;
 	case CD_MEDIA_TYPE_DVD_PLUS_R:
 		return TRUE;
 	case CD_MEDIA_TYPE_DVD_PLUS_RW:
-		*needs_blank = TRUE;
 		return TRUE;
 	}
 
@@ -435,19 +488,19 @@ media_type_matches (CDMediaType type, gboolean *needs_blank)
 
 static CDMediaType
 cd_recorder_wait_for_insertion (CDRecorder *cdrecorder, CDDrive *drive,
-		gboolean *needs_blank)
+				gboolean *needs_blank)
 {
 	CDMediaType type;
 	gboolean reload;
 
 	reload = FALSE;
-	type = cd_drive_get_media_type (drive);
+	type = cd_drive_get_media_type_and_rewritable (drive, needs_blank);
 
 	if (type == CD_MEDIA_TYPE_ERROR) {
 		reload = TRUE;
 	}
 
-	while (!media_type_matches (type, needs_blank)) {
+	while (!media_type_matches (type)) {
 		gboolean res, busy_cd;
 
 		busy_cd = (type == CD_MEDIA_TYPE_BUSY);
@@ -468,7 +521,7 @@ cd_recorder_wait_for_insertion (CDRecorder *cdrecorder, CDDrive *drive,
 			return CD_MEDIA_TYPE_ERROR;
 		}
 
-		type = cd_drive_get_media_type (drive);
+		type = cd_drive_get_media_type_and_rewritable (drive, needs_blank);
 		reload = FALSE;
 		if (type == CD_MEDIA_TYPE_UNKNOWN
 				|| type == CD_MEDIA_TYPE_ERROR) {
@@ -506,7 +559,7 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 	}
 
 	type = cd_recorder_wait_for_insertion (cdrecorder,
-			drive, &needs_blank);
+					       drive, &needs_blank);
 
 	if (type == CD_MEDIA_TYPE_ERROR) {
 		cdrecorder->priv->result = RESULT_CANCEL;
